#include "npc.h"
#include "dialogs.h"
#include "game.h"
#include "map.h"
#include "config.h"
#include <gint/display.h>
#include <gint/keyboard.h> /*debug*/

#include <stdint.h>
#include <stdlib.h>
#include <math.h>


extern bopti_image_t demo_PNJ_img;


/* the color of the text to go to the next dialog phase */
/* it improves readability to have somathing lighter */
#if defined(FXCG50)
    #define PATH_COLOR  C_RED
#else
    #define PATH_COLOR  C_BLACK
#endif


NPC *npcRPG;
uint32_t nbNPC = 0;

float length( float x, float y )
{
    return sqrtf( x*x+y*y );
}

void update_npc( [[maybe_unused]] Game *game)
{
    for( uint32_t u=0; u<nbNPC; u++ )
    {
        /* if the NPC has a path to follow AND is not currently in pause */
        /* (talking with the player) */
        if (npcRPG[u].hasPath==1 && npcRPG[u].paused==false)
        {
            float vecX = (float) (npcRPG[u].xpath[ npcRPG[u].currentPoint ] +
                                 npcRPG[u].x) - npcRPG[u].curx;
            float vecY = (float) (npcRPG[u].ypath[ npcRPG[u].currentPoint ] + 
                                 npcRPG[u].y) - npcRPG[u].cury;
            float vecN = length(vecX, vecY);

            if (vecN>0.5f)
            {
                vecX /= vecN*2.0;
                vecY /= vecN*2.0;
            }
            else
            {
                npcRPG[u].currentPoint++;
                npcRPG[u].currentPoint = npcRPG[u].currentPoint % npcRPG[u].path_length;            
            }

            npcRPG[u].curx += vecX;
            npcRPG[u].cury += vecY;

        }
    }

}

void reload_npc(Game *game)
{
    if (npcRPG!=NULL)
    {
        free(npcRPG);
        npcRPG = NULL;

    }
    
    nbNPC = 0;


    for (uint32_t u=0; u<game->map_level->nbextradata; u++) //uint pour enlever un warning
    {
        ExtraData *Data = &game->map_level->extradata[u];

        if (strcmp(Data->type, "NPC")==0)   /* the current data is a NPC */
        {
            nbNPC++;
        }
    }

    npcRPG = (NPC*) malloc( nbNPC * sizeof(NPC) );
    int currentNPC=0;

    for (uint32_t u=0; u<game->map_level->nbextradata; u++) //uint pour enlever un warning
    {
        ExtraData *Data = &game->map_level->extradata[u];

        if (strcmp(Data->type, "NPC")==0)   /* the current data is a NPC */
        {
            npcRPG[currentNPC].curx = (float) Data->x;
            npcRPG[currentNPC].cury = (float) Data->y;
            npcRPG[currentNPC].x = Data->x;
            npcRPG[currentNPC].y = Data->y;
            npcRPG[currentNPC].dialogID = Data->dialogID;
            npcRPG[currentNPC].currentPoint = 1;       
            npcRPG[currentNPC].hasPath = Data->hasPath;
            npcRPG[currentNPC].path_length = Data->path_length;
            npcRPG[currentNPC].xpath = Data->xpath;
            npcRPG[currentNPC].ypath = Data->ypath;
            npcRPG[currentNPC].paused = false;
            currentNPC++;
        }
    }
}

void npc_draw(Game *game) {
    Player *pl = &game->player;

    for (uint32_t u=0; u<nbNPC; u++) //uint pour enlever un warning
    {    
        NPC *Data = &npcRPG[u];
        
        /* TODO : This is for debugging purpose, JUste to render the path */
        /* to be followed by the NPC when this will be implemented */
        #if DEBUGMODE
            if (Data->hasPath==1)           /* this NPC has a trajectory */
            {
                int NbPoints = Data->path_length+1;
                for(int v=0; v<NbPoints; v++)
                {
                    
                    int16_t deltaX1=((int16_t) (Data->x + 
                                    Data->xpath[v % NbPoints]) * PXSIZE)
                                    -(int16_t) pl->wx;
                    int16_t deltaY1=((int16_t) (Data->y +
                                    Data->ypath[v % NbPoints]) * PXSIZE)
                                    -(int16_t) pl->wy;
                    int16_t deltaX2=((int16_t) (Data->x + 
                                    Data->xpath[(v+1) % NbPoints]) * PXSIZE)
                                    -(int16_t) pl->wx;
                    int16_t deltaY2=((int16_t) (Data->y + 
                                    Data->ypath[(v+1) % NbPoints]) * PXSIZE)
                                    -(int16_t) pl->wy;
                
                    dline( pl->px + deltaX1, pl->py + deltaY1,
                        pl->px + deltaX2, pl->py + deltaY2,
                        PATH_COLOR);                   
                }
            }
        #endif // DEBUGMODE

        int16_t delX=((int16_t) (Data->curx * PXSIZE))-(int16_t) pl->wx;
        int16_t delY=((int16_t) (Data->cury * PXSIZE))-(int16_t) pl->wy;
        dimage( pl->px-P_WIDTH/2+delX, pl->py-P_HEIGHT/2+delY, &demo_PNJ_img);      
    }
}




void OLD_npc_draw(Game *game) {
    Player *player = &game->player;

    for (uint32_t u=0; u<game->map_level->nbextradata; u++) //uint pour enlever un warning
    {
        ExtraData *Data = &game->map_level->extradata[u];
    

        if (strcmp(Data->type, "NPC")==0)   /* the current data is a NPC */
        {

            /* TODO : This is for debugging purpose, JUste to render the path */
            /* to be followed by the NPC when this will be implemented */

            #if DEBUGMODE

                if (Data->hasPath==1)           /* this NPC has a trajectory */
                {
                    int NbPoints = Data->path_length+1;
                    for(int v=0; v<NbPoints; v++)
                    {

                        
                        int16_t deltaX1=((int16_t) (Data->x + Data->xpath[v % NbPoints]) * PXSIZE)-(int16_t) player->wx;
                        int16_t deltaY1=((int16_t) (Data->y + Data->ypath[v % NbPoints]) * PXSIZE)-(int16_t) player->wy;

                        int16_t deltaX2=((int16_t) (Data->x + Data->xpath[(v+1) % NbPoints]) * PXSIZE)-(int16_t) player->wx;
                        int16_t deltaY2=((int16_t) (Data->y + Data->ypath[(v+1) % NbPoints]) * PXSIZE)-(int16_t) player->wy;
                    
                        dline( player->px + deltaX1, player->py + deltaY1,
                            player->px + deltaX2, player->py + deltaY2,
                            PATH_COLOR);                   
                    }
                }

            #endif // DEBUGMODE

            int16_t deltaX=((int16_t) (Data->x * PXSIZE))-(int16_t) player->wx;
            int16_t deltaY=((int16_t) (Data->y * PXSIZE))-(int16_t) player->wy;
            dimage( player->px-P_WIDTH/2+deltaX,
                    player->py-P_HEIGHT/2+deltaY,
                    &demo_PNJ_img);
        
        }

    }

    
}

